
#include "svt.h"
#include "JAPI.h"
#include "trakr.h"

#include "audio.h"

#include "Images/PlayCircle1.o.h"
#include "Images/PlayCircle2.o.h"
#include "Images/PlayTriangle1.o.h"
#include "Images/PlayTriangle2.o.h"
#include "Images/PlaySquare.o.h"
#include "Images/Spec.o.h"
#include "Images/Test1.o.h"
#include "Images/Test2.o.h"

bool Test_CarPushButton();
bool Test_Config( );
bool Test_Motors( );
bool Test_ADCMotors( );
bool Test_Controls( );
bool Test_FileIO( );
bool Test_OSD( );
bool Test_DefaultImages( );
bool Test_Direct_OSD();
bool Test_Timers();
bool Test_ADCs(  );
bool Test_Audio(  );
bool Test_LED( );
bool Test_MoveRectangle();

void Start()
{
  OpenGraphics();
}


void End()
{
  CloseGraphics();
}

bool Test_LED( )
{
  SetLED( 1 );
  Sleep( 1000 );
  SetLED( 0 );
  Sleep( 1000 );
  SetLED( 1 );
  Sleep( 1000 );
  SetLED( 0 );
  Sleep( 1000 );
  return true;
}


bool Run()
{
  Log( "START TEST" );

  ClearScreen();
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "TRAKR" );
  Show();

  Sleep( 1000 );

  Test_LED( );

  Test_MoveRectangle();

  Test_DefaultImages();

  Test_Config( );

  Test_FileIO( );

  Test_CarPushButton();

  Test_OSD( );

  Test_Audio();

  Test_Motors( );

  Test_Timers();

  Test_ADCs(  );

  Test_Controls( );

  // Test_Motors( );

  Test_ADCMotors( );

/*
   Test_Direct_OSD();

   Sleep( 1000 );

*/
  Log( "  Frame Count %d", SVT_Graphics_GetFrameCount() );

  Log( "END TEST" );

  return false;
}


bool Test_MoveRectangle()
{
   Color ColorTP;
   int s =0;
  //place a shape on the screen:
  ColorTP.R=0x00;
  ColorTP.G=0x00;
  ColorTP.B=0x7F;
  ColorTP.Transparent=0x0;

  SetLineWidth( 10 );
  
  DrawRectangle( 0, 0, 60, 60, ColorTP );
  
  ColorTP.R=0x7F;
  ColorTP.G=0x00;
  ColorTP.B=0x00;
  ColorTP.Transparent=0x0;

  DrawRectangle( 20, 20, 50, 50, ColorTP );
  //move a portion of the screen
  for (s=0;s<50;s++)
  {
    Sleep(100);
    ScrollRectangle(s+5,s+5,s+15,s+15,1,1);
    Show();
  }
  return true;
}


bool Test_CarPushButton()
{
	int i;

  ClearScreen();
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "CAR PUSH BUTTON" );
  Show();

  Log( "CAR PUSH BUTTON" );

  OpenMotors();

  for ( i = 0; i < 100; i++ ) {

	if ( GetCarPushButton() ) {
      SetLeftMotor( 10000 );
      SetRightMotor( 10000 );
	}
    Sleep( 100 );
    SetLeftMotor( 0 );
    SetRightMotor( 0 );
  }
  CloseMotors();

  return false;
}


#define TEST_AUDIO_FILENAME "A:\\Test\\Audio.wav"

bool Test_Audio()
{
	int i;

  ClearScreen();
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "AUDIO - NEW" );
  Show();
  Log( "Audio" );

  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "REC" );
  Show();

  if ( !StartAudioRecording( TEST_AUDIO_FILENAME ) ) {
    Log(   "  Start Recording Failed" );
    return false;
  }

  ResetTimer();
  while( ReadTimer() < 2000 ) {
	  Sleep( 10 );
	  if ( !WriteAudioData( ) ) {
		Log( "  Recording Failed" );
		break;
	  }
  }
  Log( "  Recording Lasted %d", ReadTimer() );

  StopAudioRecording();

  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "PLAY" );
  Show();


  for ( i = 0; i < 2; i++ ) {
	  Log( "  Playback %d", i );
	  StartAudioPlayback( TEST_AUDIO_FILENAME );
	  ResetTimer();
	  while( IsAudioPlaying( ) && ReadTimer() < 15000 ) {
		  Sleep( 100 );
	  }
	  Log( "  Playback Lasted %d", ReadTimer() );
  }

//  JAPI_CloseRfAudio();
//  JAPI_CloseRfVideo();
  Log( "  Audio Done");

  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "DONE" );
  Show();

  ClearScreen();
  Show();


  //extern bool SVT_WAV_Play(const char * pFiLename, TAG_HANDLE* pWave);
  //extern bool SVT_WAV_Is_Busy(TAG_HANDLE *Handle);

  return true;
}

bool Test_ADCs(  )
{
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "ADCS" );
  Show();

  Log( "Testing ADCs" );
  Log( "  Battery Voltage: %d",  GetBatteryVoltage() );
  Log( "  Left Motor Current: %d", GetLeftMotorCurrent() );
  Log( "  Right Motor Current: %d",  GetRightMotorCurrent() );

  Sleep( 1000 );

  int i;
  int s[ 8 ];
  for ( i = 0; i < 8; i++ ) {
	  s[ i ] = ReadADC( i );
	  Log( "  ACD %d: %d",  i, s[ i ] );
  }

  DrawText( 5, 5, "Testing ADCs" );
  for ( i = 0; i < 8; i++ )
	  DrawText( 5 + ( i % 4 ) * 40, 30 + ( i / 4 ) * 25, "%d", s[i] );
  Show();

  Sleep( 2000 );

  return true;
}


bool Test_Timers()
{
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "Timers" );
  Show();

  Log( "Testing Timers" );

  int i = 0;

  ResetTimer();
  Sleep( 10 );
  Log( "  10ms Sleep: Timer at %d", ReadTimer() );

  ResetTimer();
  Sleep( 1000 );
  Log( "  1000ms Sleep: Timer at %d", ReadTimer() );

  for ( i = 0; i < 10; i++ ) {
	ResetTimer();
    Sleep( 100 );
    Log( "  100ms Sleep %d: Timer at %d", i, ReadTimer() );
  }

  return true;
}


unsigned int Spec_Buffer[ 40*1024 ];
//
//bool Test_Direct_OSD()
//{
//  JAPI_OpenRfVideo();
//  SVT_Memory_Copy((char *)Spec_Buffer,_binary_Images_Test1_bmp_start, _binary_Images_Test1_bmp_end-_binary_Images_Test1_bmp_start );
//  JAPI_SendRfVideo((unsigned long*)Spec_Buffer,_binary_Images_Test1_bmp_end-_binary_Images_Test1_bmp_start);
//  Sleep( 2000 );
//  SVT_Memory_Copy((char *)Spec_Buffer,_binary_Images_Test2_bmp_start, _binary_Images_Test2_bmp_end-_binary_Images_Test2_bmp_start );
//  JAPI_SendRfVideo((unsigned long*)Spec_Buffer,_binary_Images_Test2_bmp_end-_binary_Images_Test2_bmp_start);
//  Sleep( 2000 );
//  JAPI_CloseRfVideo();
//  return true;
//}

bool Test_DefaultImages( )
{
	  Color ColorTP;
	  ColorTP.R=0x7F;
	  ColorTP.G=0;
	  ColorTP.B=0x00;
	  ColorTP.Transparent=0;

	  ClearScreen();
	  DrawText( 5, 5, "Testing" );
	  DrawText( 5, 30, "Def. Images" );
	  Show();

	  Log( "Testing Default Images" );

	  Sleep( 1000 );

	  // Load Em
	  ResetTimer();
	  RegisterDefaultImages( );
	  Log( "Loaded %d", ReadTimer() );
	  Log( "   IDs: %d %d %d %d", Image0, Image1, Image2, Image3 );

	  Sleep( 1000 );

	  Log( "  Drawing Bitmaps");
	  ResetTimer();
	  DrawImage( Image0, 10, 10, ColorTP );
	  Log( "      Draw 0 %d", ReadTimer() );
	  ResetTimer();
	  DrawImage( Image1, 20, 20, ColorTP );
	  Log( "      Draw 1 %d", ReadTimer() );
	  ResetTimer();
	  DrawImage( Image2, 30, 30, ColorTP );
	  Log( "      Draw 2 %d", ReadTimer() );
	  ResetTimer();
	  DrawImage( Image3, 40, 40, ColorTP );
	  Log( "      Draw 3 %d", ReadTimer() );
	  Show();

	  Log( "  Done Testing Default Images" );

	  Sleep( 1000 );

	  return true;

}

bool Test_OSD()
{
  Color ColorTP;
  ColorTP.R=0x7F;
  ColorTP.G=0;
  ColorTP.B=0x00;
  ColorTP.Transparent=0;

  ClearScreen();
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "Graphics" );
  Show();

  Log( "Testing Graphics" );

  Sleep( 1000 );

  int s;
    for ( s = 0; s < 10; s++ ) {
	  ResetTimer();
	  Log( "  Writing %d", s );
	  ClearScreen();
	  DrawText( 5, 30, "%d", s );
	  Show();
	  Log( "    UI Time %d", ReadTimer() );
	  Sleep( 1000 );
    }

    for ( s = 0; s < 10; s++ ) {
	  ResetTimer();
	  Log( "  Writing %d", s );
	  ClearScreen();
	  DrawText( 5, 30, "%d", s );
	  Show();
	  Log( "    UI Time %d", ReadTimer() );
	  Sleep( 100 );
    }


  ResetTimer();
  Log( "  Writing Strings" );
  ClearScreen();
  DrawText( 5, 30, "Test String" );
  Show();
  Log( "    Format and Show %d", ReadTimer() );
  Sleep( 1000 );
  ResetTimer();
  ClearScreen();
  DrawText( 5, 30, "Test %d", ReadTimer() );
  Show();
  Log( "    Number and Show %d", ReadTimer() );
  Sleep( 1000 );
  ResetTimer();
  ClearScreen();
  DrawText( 5, 30, "Test %s", "STRING" );
  Show();
  Log( "    String and Number %d", ReadTimer() );
  Sleep( 1000 );
  ResetTimer();
  ClearScreen();
  DrawText( 5, 30, "%s", "!@#$%^&*()[]{}.," );
  Show();
  Log( "    Chars and Number %d", ReadTimer() );
  Sleep( 1000 );

  ColorTP.R=0xFF;
  ColorTP.G=0xFF;
  ColorTP.B=0xFF;
  ColorTP.Transparent=0x00;
  ClearScreen();
  SetTextColor( ColorTP );
  DrawText( 5, 60, "Trans 0" );
  ResetTimer();
  ColorTP.G=0x00;
  ColorTP.R=0xFF;
  ColorTP.B=0x00;
  ColorTP.Transparent=0x00;
  SetTextColor( ColorTP );
  DrawText( 5, 30, "RED" );
  ColorTP.R=0x00;
  ColorTP.G=0xFF;
  SetTextColor( ColorTP );
  DrawText( 60, 30, "GREEN" );
  ColorTP.G=0x00;
  ColorTP.B=0xFF;
  SetTextColor( ColorTP );
  DrawText( 120, 30, "BLUE" );
  Show();
  Log( "    Colors %d", ReadTimer() );
  Sleep( 2000 );

  ColorTP.R=0xFF;
  ColorTP.G=0xFF;
  ColorTP.B=0xFF;
  ColorTP.Transparent=0x00;
  ClearScreen();
  SetTextColor( ColorTP );
  DrawText( 5, 60, "Trans 3F" );
  ResetTimer();
  ColorTP.G=0x00;
  ColorTP.R=0x7F;
  ColorTP.B=0x00;
  ColorTP.Transparent=0x3F;
  SetTextColor( ColorTP );
  DrawText( 5, 30, "RED" );
  ColorTP.R=0x00;
  ColorTP.G=0x7F;
  SetTextColor( ColorTP );
  DrawText( 60, 30, "GREEN" );
  ColorTP.G=0x00;
  ColorTP.B=0x7F;
  SetTextColor( ColorTP );
  DrawText( 120, 30, "BLUE" );
  Show();
  Log( "    Colors %d", ReadTimer() );
  Sleep( 2000 );

  ColorTP.R=0xFF;
  ColorTP.G=0xFF;
  ColorTP.B=0xFF;
  ColorTP.Transparent=0x00;
  ClearScreen();
  SetTextColor( ColorTP );
  DrawText( 5, 60, "Trans 7F" );
  ResetTimer();
  ColorTP.G=0x00;
  ColorTP.R=0x7F;
  ColorTP.B=0x00;
  ColorTP.Transparent=0x7F;
  SetTextColor( ColorTP );
  DrawText( 5, 30, "RED" );
  ColorTP.R=0x00;
  ColorTP.G=0x7F;
  SetTextColor( ColorTP );
  DrawText( 60, 30, "GREEN" );
  ColorTP.G=0x00;
  ColorTP.B=0x7F;
  SetTextColor( ColorTP );
  DrawText( 120, 30, "BLUE" );
  Show();
  Log( "    Colors %d", ReadTimer() );
  Sleep( 2000 );

  ColorTP.R=0xFF;
  ColorTP.G=0xFF;
  ColorTP.B=0xFF;
  ColorTP.Transparent=0x00;
  SetTextColor( ColorTP );

  ClearScreen();
  DrawText( 5, 30, "Rectangles" );
  Show();
  Sleep( 1000 );

  ResetTimer();
  Log( "  Rectangles" );

  ColorTP.R=0x7F;
  ColorTP.G=0x7F;
  ColorTP.B=0x7F;
  ColorTP.Transparent=0x0;

  SetLineWidth( 10 );
  DrawRectangle( 0, 0, 60, 60, ColorTP );
  Log( "    Rect 1 " );

  ColorTP.R=0x7F;
  ColorTP.G=0x7F;
  ColorTP.B=0x00;
  ColorTP.Transparent=0x0;

  DrawRectangle( 10, 10, 50, 50, ColorTP );
  Log( "    Rect 2 " );

  ClearRectangle( 20, 20, 30, 30 );
  Log( "    Clear Rect " );

  Show();
  Log( "    Show " );

  Sleep( 3000 );

  ClearScreen();
  DrawText( 5, 30, "Bitmaps" );
  Show();
  Sleep( 1000 );

  ResetTimer();
  Log( "  Loading Test Bitmaps" );
  // Send some bitmaps
  OpenImageRegister();
  Log( "    Pack Open", ReadTimer() );
  RegisterImage(_binary_PlayCircle1_bin_start,
                     _binary_PlayCircle1_bin_end-\
                     _binary_PlayCircle1_bin_start);
  Log( "    PlayCircle %d", s, s, ReadTimer() );
  RegisterImage(_binary_PlayTriangle1_bin_start,
                     _binary_PlayTriangle1_bin_end-\
                     _binary_PlayTriangle1_bin_start);
  Log( "    PlayTriangle1 %d", s, s, ReadTimer() );
  RegisterImage(_binary_PlaySquare_bin_start,
                     _binary_PlaySquare_bin_end-\
                     _binary_PlaySquare_bin_start);
  Log( "    PlaySquare %d", s, s, ReadTimer() );
  RegisterImage(_binary_Images_Spec_bmp_start,
                     _binary_Images_Spec_bmp_end-\
                     _binary_Images_Spec_bmp_start);
  CloseImageRegister();
  Log( "    SENT %d", s, s, ReadTimer() );

  Log( "  Drawing Bitmaps");
  for ( s = 0; s < 10; s++ ) {
	ResetTimer();
	ClearScreen();
    Log( "    Iteration %d", s );
    DrawImage( 1, s * 10, 50, ColorTP );
    Log( "      Draw 1 %d", ReadTimer() );
    DrawImage( 2, s * 10, 90, ColorTP );
    Log( "      Draw 2 %d", ReadTimer() );
    Show();
    Log( "      Show %d", ReadTimer() );
    if ( ReadTimer() > 10 )
        Log( "      Two Bitmaps + Show > 10ms" );

    Sleep( 1000 );
  }

  Log( "  Done Testing Graphics" );

  return true;
}

bool Test_Config( )
{
  Log( "Testing Config" );
  Log( "  char size : %d", sizeof( char ) );
  Log( "  short size : %d", sizeof( short ) );
  Log( "  int size : %d", sizeof( int ) );
  Log( "  long size : %d", sizeof( long ) );
  return true;
}

#define TEST_FILENAME "A:\\Test\\FileTest.tmp"

bool Test_FileIO(  )
{
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "FileIO" );
  Show();

  Log( "Testing FileIO" );
  int i;
  DeleteFile( TEST_FILENAME );
  SVT_FileHandle fh = CreateFile( TEST_FILENAME );
  uint32 written;
  for ( i = 0; i < 10; i++ )
    written = WriteFile( fh, "Trakr\n", 6 );
  if ( written !=6 )
   Log( "  Wrong length %d != 6", written );
  written = WriteFile( fh, "Spy\n", 4 );
  CloseFile( fh );
  Log( "  File Written" );

  uint32 read;
  char buffer[ 100 ];
  fh = SVT_File_Open( TEST_FILENAME );
  Log( "  Reading File" );
  for ( i = 0; i < 10; i++ ) {
    read = ReadFile( fh, buffer, 6 );
    buffer[ 5 ] = 0;
    if ( read != 6 || !SVT_Str_Cmp( buffer, "Trakr") )
      Log( "  Read Error Size %d != 6, %s != \"Trakr\"", read, buffer );
  }
  read = ReadFile( fh, buffer, 4 );
  buffer[ 3 ] = 0;
  if ( read != 4 || !SVT_Str_Cmp( buffer, "Spy") )
    Log( "  Read Error Size %d != 4, %s != \"Spy\"", read, buffer );
  CloseFile( fh );

  Log( "  End Testing FileIO" );
  return true;
}

bool Test_ADCMotors(  )
{
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "ADC Rapid" );
  Show();

  Log( "Testing Motor Current" );
  Sleep( 1000 );

  ClearScreen();

  int s[ 8 ];

  int i;
  int j;
  for ( j = 0; j < 50; j++ ) {
	  for ( i = 0; i < 8; i++ ) {
		  s[ i ] = ReadADC( i );
	  }
	  Log( "  Key: %x ACD %d, %d, %d, %d, %d, %d, %d, %d", GetRemoteKeys(), s[ 0 ], s[ 1 ], s[ 2 ], s[ 3 ], s[ 4 ], s[ 5 ], s[ 6 ], s[ 7 ] );
	  Sleep( 100 );
  }

  j = 0;
  while ( j < 5 )
  {
	  for ( i = 0; i < 8; i++ ) {
		  s[ i ] = ReadADC( i );
		  Log( "  ACD %d, %d, %d, %d, %d, %d, %d, %d", s[ 0 ], s[ 1 ], s[ 2 ], s[ 3 ], s[ 4 ], s[ 5 ], s[ 6 ], s[ 7 ] );
	  }

	  ClearScreen();
	  DrawText( 5, 5, "Testing ADCs" );
	  for ( i = 0; i < 8; i++ )
		  DrawText( 5 + ( i % 4 ) * 40, 30 + ( i / 4 ) * 25, "%d", s[i] );
	  Show();

      Sleep( 1000 );

      if ( GetRemoteKeys() == 0 )
    	  j++;
      else
    	  j = 0;
  }
  return true;
}

bool Test_Motors(  )
{
  int i;
  int lc;
  int rc;

  ClearScreen();
  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "Motors" );
  Show();

  Log( "Testing Motors" );
  Sleep( 1000 );

  Log( "  Motors Open" );
  OpenMotors();


  Log( "New API" );

  SetLeftMotor( 0 );
  SetRightMotor( 0 );
  Sleep( 1000 );

  for ( i = 0; i < 10000; i += 100 ) {
	  SetLeftMotor( i );
	  SetRightMotor( i );
	  Sleep( 100 );
  }

  for ( i = 10000; i > -10000; i -= 100 ) {
	  SetLeftMotor( i );
	  SetRightMotor( i );
	  Sleep( 100 );
  }

  SetLeftMotor( 0 );
  SetRightMotor( 0 );
  Sleep( 1000 );
  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  Log( "    LC %d RC %d", lc, rc );
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Sleep( 1000 );

  SetLeftMotor( 10000 );
  Log( "  Motor Set Left 100" );
  Sleep( 1000 );
  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  Log( "    LC %d RC %d", lc, rc );
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();

  SetLeftMotor( -10000 );
  Log( "  Motor Set Left -100" );
  Sleep( 1000 );
  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );

  Log( "  Motor Set Left 0" );
  SetLeftMotor( 0 );
  Sleep( 1000 );

  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );

  Log( "  Motor Set Right 100" );
  SetRightMotor( 10000 );
  Sleep( 1000 );
  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );

  Log( "  Motor Set Right -100" );
  SetRightMotor( -10000 );
  Sleep( 1000 );
  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );


  Log( "  Motor Set Right 0" );
  SetRightMotor( 0 );
  Sleep( 1000 );

  Log( "  Motor Set Left & Right 100" );
  SetRightMotor( 10000 );
  SetLeftMotor( 10000 );
  Sleep( 1000 );

  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );


  Log( "  Motor Set Left & Right -100" );
  SetRightMotor( -10000 );
  SetLeftMotor( -10000 );
  Sleep( 1000 );

  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );

  Log( "  Motor Set Left & Right 0" );
  SetRightMotor( 0 );
  SetLeftMotor( 0 );

  Sleep( 1000 );

  lc = GetLeftMotorCurrent();
  rc = GetRightMotorCurrent();
  ClearRectangle( 5, 80, 159, 119  );
  DrawText( 5, 80, "L %d", lc );
  DrawText( 80, 80, "R %d", rc );
  Show();
  Log( "    LC %d RC %d", lc, rc );

  SVT_Motor_Close();

  return true;
}

bool Test_Controls(  )
{
  Log( "Testing Controls" );

  SVT_Motor_Open();

  Color color;
  color.R=0;
  color.G=0;
  color.B=0x80;
  color.Transparent=0;

  DrawText( 5, 5, "Testing" );
  DrawText( 5, 30, "Controls" );
  Show();
  Sleep( 1000 );

  ClearScreen();
  DrawText( 30, 5, "R B" );
  DrawImage( 2, 100, 80, color);
  Show();

  Sleep( 1000 );

  int count = 0;

  while ( !( SVT_Key_GetStatus() & SVT_KEY_RIGHT_BACK ) && ( count++ < 50 ))
    Sleep( 100 );

  if ( count >= 50 )
    Log( "  Failed to Receive Right Back" );
  else
    Log( "  Right Back" );

  ClearScreen();
  DrawText( 30, 5, "R F" );
  DrawImage( 2, 100, 30, color);
  Show();

  count = 0;

  while ( !( SVT_Key_GetStatus() & SVT_KEY_RIGHT_FORWARD ) && ( count++ < 50 ))
    Sleep( 100 );

  if ( count >= 50 )
    Log( "  Failed to Receive Right Forward" );
  else
    Log( "  Right Forward" );

  // LEFT

  ClearScreen();
  DrawText( 30, 5, "L B" );
  DrawImage( 2, 20, 80, color);
  Show();

  count = 0;

  while ( !( SVT_Key_GetStatus() & SVT_KEY_LEFT_BACK ) && ( count++ < 50 ))
    Sleep( 100 );

  if ( count >= 50 )
    Log( "  Failed to Receive Left Back" );
  else
    Log( "  Left Back" );

  ClearScreen();
  DrawText( 30, 5, "L F" );
  DrawImage( 2, 20, 30, color);
  Show();

  count = 0;

  while ( !( SVT_Key_GetStatus() & SVT_KEY_LEFT_FORWARD ) && ( count++ < 50 ))
    Sleep( 100 );

  if ( count >= 50 )
    Log( "  Failed to Receive Left Forward" );
  else
    Log( "  Left Forward" );

  ClearScreen();
  DrawText( 30, 5, "0ff" );
  Show();

  count = 0;

  while ( ( SVT_Key_GetStatus() ) && ( count++ < 50 ))
     Sleep( 100 );

   if ( count >= 50 )
     Log( "  Failed to Return to no key" );
   else
     Log( "  No Keys" );

   SVT_Motor_Open();

  return true;
}
